@@ -29,7 +29,7 @@
 #' @examples
 #' \dontrun{
 #' if (require("microbenchmark") && has_lahman()) {
-#' lahman_local <- lahman_srcs("df", "dt")
+#' lahman_local <- lahman_srcs("df", "sqlite")
 #' teams <- lapply(lahman_local, function(x) x %>% tbl("Teams"))
 #'
 #' compare_tbls(teams, function(x) x %>% filter(yearID == 2010))
@@ -82,7 +82,7 @@ bench_tbls <- function(tbls, op, ..., times = 10) {
 #' @rdname bench_compare
 compare_tbls <- function(tbls, op, ref = NULL, compare = equal_data_frame, ...) {
   if (length(tbls) < 2 && is.null(ref)) {
-    stop("Need at least two srcs to compare", call. = FALSE)
+    testthat::skip("Need at least two srcs to compare")
   }
   if (!requireNamespace("testthat", quietly = TRUE)) {
     stop("Please install the testthat package", call. = FALSE)
