@@ -3,6 +3,8 @@
 
 namespace dplyr{
 
+    CharacterVector get_uniques( const CharacterVector& left, const CharacterVector& right) ;
+
     template <int LHS_RTYPE, int RHS_RTYPE>
     class JoinVisitorImpl : public JoinVisitor, public comparisons_different<LHS_RTYPE, RHS_RTYPE>{
     public:
@@ -15,7 +17,9 @@ namespace dplyr{
         typedef boost::hash<LHS_STORAGE> LHS_hasher ;
         typedef boost::hash<RHS_STORAGE> RHS_hasher ;
 
-        JoinVisitorImpl( LHS_Vec left_, RHS_Vec right_ ) : left(left_), right(right_){}
+        JoinVisitorImpl( LHS_Vec left_, RHS_Vec right_ ) : left(left_), right(right_){
+          check_attribute_compatibility(left, right) ;
+        }
 
         size_t hash(int i) ;
 
@@ -34,14 +38,6 @@ namespace dplyr{
         inline SEXP subset( const std::vector<int>& indices );
         inline SEXP subset( const VisitorSetIndexSet<DataFrameJoinVisitors>& set ) ;
 
-        inline void print(int i){
-            if( i >= 0){
-                Rcpp::Rcout << left[i] << std::endl ;
-            } else {
-                Rcpp::Rcout << right[-i-1] << std::endl ;
-            }
-        }
-
         LHS_Vec left ;
         RHS_Vec right ;
         LHS_hasher LHS_hash_fun ;
@@ -49,6 +45,35 @@ namespace dplyr{
 
     } ;
 
+    template <typename Visitor>
+    class Subsetter {
+    public:
+        typedef typename Visitor::Vec Vec ;
+
+        Subsetter( const Visitor& v_) : v(v_){} ;
+
+        inline SEXP subset( const std::vector<int>& indices ) {
+          int n = indices.size() ;
+          Vec res = no_init(n) ;
+          for( int i=0; i<n; i++) {
+              res[i] = v.get(indices[i]) ;
+          }
+          return res ;
+        }
+
+        inline SEXP subset( const VisitorSetIndexSet<DataFrameJoinVisitors>& set ) {
+            int n = set.size() ;
+            Vec res = no_init(n) ;
+            VisitorSetIndexSet<DataFrameJoinVisitors>::const_iterator it=set.begin() ;
+            for( int i=0; i<n; i++, ++it) {
+                res[i] = v.get(*it) ;
+            }
+            return res ;
+        }
+    private:
+        const Visitor& v ;
+    } ;
+
     template <int RTYPE>
     class JoinVisitorImpl<RTYPE,RTYPE> : public JoinVisitor, public comparisons<RTYPE>{
     public:
@@ -71,394 +96,301 @@ namespace dplyr{
         }
 
         inline SEXP subset( const std::vector<int>& indices ) {
-            int n = indices.size() ;
-            Vec res = no_init(n) ;
-            for( int i=0; i<n; i++) {
-                res[i] = get(indices[i]) ;
-            }
-            return res ;
-        }
-        inline SEXP subset( const VisitorSetIndexSet<DataFrameJoinVisitors>& set ) {
-            int n = set.size() ;
-            Vec res = no_init(n) ;
-            VisitorSetIndexSet<DataFrameJoinVisitors>::const_iterator it=set.begin() ;
-            for( int i=0; i<n; i++, ++it) {
-                res[i] = get(*it) ;
-            }
+            RObject res = Subsetter<JoinVisitorImpl>(*this).subset(indices ) ;
+            copy_most_attributes(res, left) ;
             return res ;
         }
 
-        inline void print(int i){
-            Rcpp::Rcout << get(i) << std::endl ;
+        inline SEXP subset( const VisitorSetIndexSet<DataFrameJoinVisitors>& set ) {
+            RObject res = Subsetter<JoinVisitorImpl>(*this).subset(set) ;
+            copy_most_attributes(res, left) ;
+            return res ;
+        }
+
+        inline STORAGE get(int i) const {
+            return i >= 0 ? left[i] : right[-i-1] ;
         }
 
     protected:
         Vec left, right ;
         hasher hash_fun ;
 
-        inline STORAGE get(int i){
-            return i >= 0 ? left[i] : right[-i-1] ;
-        }
-
     } ;
 
-    class JoinStringOrderer {
+    class JoinFactorFactorVisitor : public JoinVisitor {
     public:
-        JoinStringOrderer( const CharacterVector& left_, const CharacterVector& right_ ) :
-            left(left_), right(right_), nleft(left.size()), nright(right.size()), n(nleft+nright), n_na(0)
-        {
-            make_orders() ;
-        }
-
-        inline int get_order(int i) const {
-            if( i == NA_INTEGER ) return NA_INTEGER ;
-            int val = (i>=0) ? orders[i] : orders[nleft-i-1] ;
-            if( val > n - n_na ) val= NA_INTEGER ;
-            return val ;
-        }
-
-    private:
-        const CharacterVector& left ;
-        const CharacterVector& right ;
-        int nleft, nright, n ;
-        IntegerVector orders ;
-        int n_na ;
-
-        inline void make_orders(){
-            CharacterVector big(n) ;
-            CharacterVector::iterator it = big.begin() ;
-            std::copy( left.begin(), left.end(), it ) ;
-            std::copy( right.begin(), right.end(), it + nleft ) ;
-            orders = CharacterVectorOrderer(big).get() ;
-            n_na = std::count( big.begin(), big.end(), NA_STRING ) ;
-        }
-
-    } ;
-
-
-    template <>
-    class JoinVisitorImpl<STRSXP,STRSXP> : public JoinVisitor, public comparisons<STRSXP>{
-    public:
-        typedef comparisons<STRSXP> Compare ;
-
         typedef CharacterVector Vec ;
-        typedef SEXP STORAGE ;
-        typedef boost::hash<int> hasher ;
 
-        JoinVisitorImpl( CharacterVector left_, CharacterVector right_ ) :
-            left(left_), right(right_), orderer(left,right)
+        JoinFactorFactorVisitor( const IntegerVector& left_, const IntegerVector& right_ ) :
+            left(left_),
+            right(right_),
+            left_levels (left.attr("levels")),
+            right_levels(right.attr("levels")),
+            uniques( get_uniques(left_levels, right_levels) ),
+            left_match ( match( left_levels, uniques) ),
+            right_match( match( right_levels, uniques) )
         {}
 
         inline size_t hash(int i){
-            return hash_fun( orderer.get_order(i) ) ;
+            return hash_fun( get(i) ) ;
         }
 
-        inline bool equal( int i, int j) {
-            return orderer.get_order(i) == orderer.get_order(j) ;
+        inline bool equal( int i, int j){
+            return get(i) == get(j) ;
         }
 
         inline SEXP subset( const std::vector<int>& indices ) {
-            int n = indices.size() ;
-            Vec res = no_init(n) ;
-            for( int i=0; i<n; i++) {
-                res[i] = get(indices[i]) ;
-            }
-            return res ;
+            return Subsetter<JoinFactorFactorVisitor>(*this).subset(indices) ;
         }
+
         inline SEXP subset( const VisitorSetIndexSet<DataFrameJoinVisitors>& set ) {
-            int n = set.size() ;
-            Vec res = no_init(n) ;
-            VisitorSetIndexSet<DataFrameJoinVisitors>::const_iterator it=set.begin() ;
-            for( int i=0; i<n; i++, ++it) {
-                res[i] = get(*it) ;
+            return Subsetter<JoinFactorFactorVisitor>(*this).subset(set) ;
+        }
+
+        inline SEXP get(int i) const {
+            if( i >= 0){
+              int pos = left[i] ;
+              return (pos == NA_INTEGER) ? NA_STRING : SEXP( uniques[left_match[pos-1] - 1] ) ;
+            } else {
+              int pos = right[-i-1] ;
+              return (pos == NA_INTEGER) ? NA_STRING : SEXP( uniques[right_match[ pos -1 ] - 1] ) ;
             }
-            return res ;
         }
 
-        inline void print(int i){
-            Rcpp::Rcout << get(i) << std::endl ;
-        }
-
-
-    protected:
-
-        CharacterVector left, right ;
-        hasher hash_fun ;
-        JoinStringOrderer orderer ;
-
-        inline STORAGE get(int i){
-            return i >= 0 ? left[i] : right[-i-1] ;
-        }
+    private:
+        IntegerVector left, right ;
+        CharacterVector left_levels, right_levels ;
+        CharacterVector uniques ;
+        IntegerVector left_match, right_match ;
+        boost::hash<SEXP> hash_fun ;
 
     } ;
 
-    class StringLessPredicate : comparisons<STRSXP>{
+    class JoinStringStringVisitor : public JoinVisitor {
     public:
-        typedef SEXP value_type ;
-        inline bool operator()( SEXP x, SEXP y){
-            return is_less(x, y) ;
+        typedef CharacterVector Vec ;
+
+        JoinStringStringVisitor( CharacterVector left_, CharacterVector right) :
+            left(left_),
+            uniques( get_uniques(left, right) ),
+            i_left( match(left, uniques) ),
+            i_right( match(right, uniques) ),
+            int_visitor( i_left, i_right),
+            p_uniques( internal::r_vector_start<STRSXP>(uniques) ),
+            p_left( i_left.begin() ),
+            p_right( i_right.begin() )
+        {}
+
+        inline size_t hash(int i) {
+          return int_visitor.hash(i) ;
         }
+        bool equal(int i, int j) {
+          return int_visitor.equal(i,j) ;
+        }
+
+        inline SEXP subset( const std::vector<int>& indices ) {
+            RObject res = Subsetter<JoinStringStringVisitor>(*this).subset(indices) ;
+            copy_most_attributes( res, left) ;
+            return res ;
+        }
+
+        inline SEXP subset( const VisitorSetIndexSet<DataFrameJoinVisitors>& set ) {
+            RObject res = Subsetter<JoinStringStringVisitor>(*this).subset(set) ;
+            copy_most_attributes(res, left) ;
+            return res ;
+        }
+
+        inline SEXP get(int i) const {
+            if( i >= 0 ){
+                return ( i_left[i] == NA_INTEGER ) ? NA_STRING : p_uniques[ p_left[i] - 1] ;
+            } else {
+                return ( i_right[-i-1] == NA_INTEGER ) ? NA_STRING : p_uniques[ p_right[-i-1] - 1] ;
+            }
+        }
+
+    private:
+        CharacterVector left ;
+        CharacterVector uniques ;
+        IntegerVector i_left, i_right ;
+        JoinVisitorImpl<INTSXP,INTSXP> int_visitor ;
+        SEXP* p_uniques ;
+        int* p_left ;
+        int* p_right ;
+
+
     } ;
 
     class JoinFactorStringVisitor : public JoinVisitor {
     public:
+        typedef CharacterVector Vec ;
+
         JoinFactorStringVisitor( const IntegerVector& left_, const CharacterVector& right_ ) :
             left(left_),
+            left_ptr(left.begin()),
+
             right(right_),
-            left_ptr(left_.begin()),
-            left_levels( left_.attr("levels") ),
-            left_factor_ptr(Rcpp::internal::r_vector_start<STRSXP>(left_levels) ),
-            right_ptr(Rcpp::internal::r_vector_start<STRSXP>(right_)),
-            orderer(left_levels, right)
+            uniques( get_uniques(left.attr("levels"), right) ) ,
+            p_uniques( internal::r_vector_start<STRSXP>(uniques) ),
+
+            i_right( match( right, uniques )),
+            int_visitor( left, i_right )
+
         {}
 
         inline size_t hash(int i){
-            return hash_fun( orderer.get_order(get_pos(i)) ) ;
+            return int_visitor.hash(i) ;
         }
 
         inline bool equal( int i, int j){
-            return orderer.get_order(get_pos(i)) == orderer.get_order(get_pos(j)) ;
-        }
-
-        inline void print(int i){
-            Rcpp::Rcout << get(i) << std::endl ;
+            return int_visitor.equal(i,j) ;
         }
 
         inline SEXP subset( const std::vector<int>& indices ) {
-            int n = indices.size() ;
-            CharacterVector res(n) ;
-            for( int i=0; i<n; i++) {
-                res[i] = get(indices[i]) ;
-            }
+            RObject res = Subsetter<JoinFactorStringVisitor>(*this).subset(indices) ;
+            // copy_most_attributes(res, left) ;
             return res ;
         }
+
         inline SEXP subset( const VisitorSetIndexSet<DataFrameJoinVisitors>& set ) {
-            int n = set.size() ;
-            CharacterVector res(n) ;
-            VisitorSetIndexSet<DataFrameJoinVisitors>::const_iterator it=set.begin() ;
-            for( int i=0; i<n; i++, ++it) {
-                res[i] = get(*it) ;
-            }
+            RObject res = Subsetter<JoinFactorStringVisitor>(*this).subset(set) ;
+            // copy_most_attributes(res, left) ;
             return res ;
         }
 
+        inline SEXP get(int i) const {
+            if( i>=0 ){
+                if( left_ptr[i] == NA_INTEGER ) return NA_STRING ;
+                return p_uniques[ left_ptr[i] - 1 ] ;
+            } else {
+                return p_uniques[ i_right[ -i-1 ] - 1] ;
+            }
+        }
+
     private:
         IntegerVector left ;
+        int* left_ptr ;
+
         CharacterVector right ;
-        IntegerVector::const_iterator left_ptr ;
-        CharacterVector left_levels ;
-        SEXP* left_factor_ptr ;
-        SEXP* right_ptr ;
-        boost::hash<int> hash_fun ;
-        JoinStringOrderer orderer ;
+        CharacterVector uniques ;
+        SEXP* p_uniques ;
+        IntegerVector i_right ;
 
-        inline SEXP get(int i){
-            if( i>=0 ){
-                if( left_ptr[i] == NA_INTEGER ) return NA_STRING ;
-                return left_factor_ptr[ left_ptr[i] - 1 ] ;
-            } else {
-                return right_ptr[ -i-1 ] ;
-            }
-        }
+        JoinVisitorImpl<INTSXP,INTSXP> int_visitor ;
 
-        inline int get_pos( int i ){
-            if( i>= 0 ) {
-                if( left_ptr[i] == NA_INTEGER ) return NA_INTEGER ;
-                return left_ptr[i] - 1 ;
-            }
-            return i ;
-        }
 
     } ;
 
     class JoinStringFactorVisitor : public JoinVisitor {
     public:
+        typedef CharacterVector Vec ;
+
         JoinStringFactorVisitor( const CharacterVector& left_, const IntegerVector& right_ ) :
             left(left_),
-            right(right_),
-            right_ptr(right_.begin()),
-            right_levels(right_.attr("levels")),
-            right_factor_ptr(Rcpp::internal::r_vector_start<STRSXP>(right_levels) ),
-            left_ptr(Rcpp::internal::r_vector_start<STRSXP>(left_)),
-            orderer(left, right_levels)
+            i_right(right_),
+            uniques( get_uniques(i_right.attr("levels"), left_) ),
+            p_uniques( internal::r_vector_start<STRSXP>(uniques) ),
+            i_left( match(left_, uniques) ),
+
+            int_visitor(i_left, i_right)
         {}
 
         inline size_t hash(int i){
-            return hash_fun( orderer.get_order(get_pos(i)) ) ;
+            return int_visitor.hash(i) ;
         }
 
         inline bool equal( int i, int j){
-            return orderer.get_order(get_pos(i)) == orderer.get_order(get_pos(j)) ;
-        }
-
-        inline void print(int i){
-            Rcpp::Rcout << get(i) << std::endl ;
+            return int_visitor.equal(i,j) ;
         }
 
         inline SEXP subset( const std::vector<int>& indices ) {
-            int n = indices.size() ;
-            CharacterVector res(n) ;
-            for( int i=0; i<n; i++) {
-                res[i] = get(indices[i]) ;
-            }
-            return res ;
+            RObject res = Subsetter<JoinStringFactorVisitor>(*this).subset(indices) ;
+            // copy_most_attributes(res, left) ;
+            return res;
         }
+
         inline SEXP subset( const VisitorSetIndexSet<DataFrameJoinVisitors>& set ) {
-            int n = set.size() ;
-            CharacterVector res(n) ;
-            VisitorSetIndexSet<DataFrameJoinVisitors>::const_iterator it=set.begin() ;
-            for( int i=0; i<n; i++, ++it) {
-                res[i] = get(*it) ;
-            }
+            RObject res = Subsetter<JoinStringFactorVisitor>(*this).subset(set) ;
+            // copy_most_attributes(res, left) ;
             return res ;
         }
 
-    private:
-        CharacterVector left ;
-        IntegerVector right ;
-        IntegerVector::const_iterator  right_ptr ;
-        CharacterVector right_levels ;
-        SEXP* right_factor_ptr ;
-        SEXP* left_ptr ;
-        boost::hash<int> hash_fun ;
-        JoinStringOrderer orderer ;
-
-        inline SEXP get(int i){
+        inline SEXP get(int i) const {
             SEXP res ;
 
             if( i>=0 ){
-                res = left_ptr[i] ;
+                res = p_uniques[ i_left[i] - 1 ] ;
             } else {
                 int index = -i-1 ;
-                if( right_ptr[index] == NA_INTEGER ) {
+                if( i_right[index] == NA_INTEGER ) {
                     res = NA_STRING ;
                 } else {
-                    res = right_factor_ptr[ right_ptr[index] - 1 ] ;
+                    res = p_uniques[ i_right[index] - 1 ] ;
                 }
             }
 
             return res ;
         }
 
-        inline int get_pos(int i) const {
-            if( i>=0 ) {
-                return i ;
-            }
-            int index = right_ptr[-i-1] ;
-            if( index == NA_INTEGER ) return NA_INTEGER ;
-            return - index ;
-        }
+    private:
+        CharacterVector left ;
+        IntegerVector i_right ;
+        CharacterVector uniques ;
+        SEXP* p_uniques ;
+        IntegerVector i_left ;
 
+        JoinVisitorImpl<INTSXP, INTSXP> int_visitor ;
 
     } ;
 
 
-    class JoinFactorFactorVisitor : public JoinVisitorImpl<INTSXP, INTSXP> {
+
+    class POSIXctJoinVisitor : public JoinVisitorImpl<REALSXP,REALSXP> {
     public:
-        typedef JoinVisitorImpl<INTSXP,INTSXP> Parent ;
+        typedef JoinVisitorImpl<REALSXP,REALSXP> Parent ;
+        POSIXctJoinVisitor( NumericVector left, NumericVector right) :
+          Parent(left, right),
+          tzone(R_NilValue)
+        {
+          RObject tzone_left  = left.attr("tzone") ;
+          RObject tzone_right = right.attr("tzone") ;
+          if( tzone_left.isNULL() && tzone_right.isNULL() ) return ;
 
-        JoinFactorFactorVisitor( const IntegerVector& left, const IntegerVector& right ) :
-            Parent(left, right),
-            left_levels(left.attr("levels")),
-            right_levels(right.attr("levels")),
-            left_levels_ptr( Rcpp::internal::r_vector_start<STRSXP>( left_levels ) ) ,
-            right_levels_ptr( Rcpp::internal::r_vector_start<STRSXP>( right_levels ) ),
-            orderer(left_levels, right_levels)
-            {}
+          if( tzone_left.isNULL() ) {
+            tzone = tzone_right ;
+          } else if( tzone_right.isNULL() ) {
+            tzone = tzone_left ;
+          } else {
+            std::string s_left  = as<std::string>( tzone_left  ) ;
+            std::string s_right = as<std::string>( tzone_right ) ;
 
-        inline size_t hash(int i){
-            return hash_fun( orderer.get_order(get_pos(i)) ) ;
-        }
-
-        void print(int i){
-            Rcpp::Rcout << get(i) << " :: " << toString<STRSXP>(get(i)) << std::endl ;
-        }
-
-        inline bool equal( int i, int j){
-            return orderer.get_order(get_pos(i)) ==  orderer.get_order(get_pos(j)) ;
-        }
-
-        inline SEXP subset( const VisitorSetIndexSet<DataFrameJoinVisitors>& set ){
-            int n = set.size() ;
-            CharacterVector res(n) ;
-
-            VisitorSetIndexSet<DataFrameJoinVisitors>::const_iterator it = set.begin() ;
-            for( int i=0; i<n; i++, ++it){
-                res[i] = get(*it) ;
+            if( s_left == s_right){
+              tzone = wrap(s_left) ;
+            } else {
+              tzone = wrap("UTC") ;
             }
-
-            return res ;
+          }
         }
 
         inline SEXP subset( const std::vector<int>& indices ){
-            int n = indices.size() ;
-            CharacterVector res(n) ;
-
-            for( int i=0; i<n; i++){
-                res[i] = get(indices[i]) ;
-            }
-
-            return res ;
+          return promote( Parent::subset( indices ) ) ;
+        }
+        inline SEXP subset( const VisitorSetIndexSet<DataFrameJoinVisitors>& set ) {
+          return promote( Parent::subset(set)) ;
         }
 
     private:
-        CharacterVector left_levels, right_levels ;
-        SEXP* left_levels_ptr ;
-        SEXP* right_levels_ptr ;
-        JoinStringOrderer orderer ;
-        boost::hash<int> hash_fun ;
+        RObject tzone ;
 
-        inline SEXP get(int i){
-            if( i >= 0 ){
-                return ( left[i] == NA_INTEGER ) ? NA_STRING : left_levels_ptr[ left[i] - 1] ;
-            } else {
-                return ( right[-i-1] == NA_INTEGER ) ? NA_STRING : right_levels_ptr[right[-i-1] - 1] ;
-            }
-        }
-
-        inline int get_pos(int i) const {
-            if( i >= 0 ){
-                if( left[i] == NA_INTEGER ) return NA_INTEGER ;
-                return left[i] - 1 ;
-            } else {
-                if( right[-i-1] == NA_INTEGER ) return NA_INTEGER ;
-                return - right[-i-1] ;
-            }
+        inline SEXP promote( NumericVector x){
+          x.attr("class") = Rcpp::CharacterVector::create("POSIXct", "POSIXt") ;
+          if( !tzone.isNULL() ){
+            x.attr("tzone") = tzone ;
+          }
+          return x ;
         }
 
     } ;
 
-    template <typename Class, typename JoinVisitorImpl>
-    class PromoteClassJoinVisitor : public JoinVisitorImpl {
-    public:
-        typedef typename JoinVisitorImpl::Vec Vec ;
-
-        PromoteClassJoinVisitor( const Vec& left, const Vec& right) : JoinVisitorImpl(left, right){}
-
-        inline SEXP subset( const std::vector<int>& indices ){
-            return promote( JoinVisitorImpl::subset( indices) ) ;
-        }
-
-        inline SEXP subset( const VisitorSetIndexSet<DataFrameJoinVisitors>& set ){
-            return promote( JoinVisitorImpl::subset( set) ) ;
-        }
-
-    private:
-        inline SEXP promote( Vec vec){
-            copy_most_attributes(vec,JoinVisitorImpl::left ) ;
-            return vec ;
-        }
-    } ;
-
-    #define PROMOTE_JOIN_VISITOR(__CLASS__)                                                   \
-    class __CLASS__ : public PromoteClassJoinVisitor<__CLASS__, JoinVisitorImpl<REALSXP,REALSXP> >{   \
-    public:                                                                                   \
-        typedef PromoteClassJoinVisitor<__CLASS__, JoinVisitorImpl<REALSXP,REALSXP> > Parent ;        \
-        __CLASS__( const NumericVector& left_, const NumericVector& right_) :                 \
-            Parent(left_, right_){}                                                           \
-    } ;
-    PROMOTE_JOIN_VISITOR(POSIXctJoinVisitor)
-
     JoinVisitor* join_visitor( SEXP, SEXP, const std::string&, const std::string&, bool warn ) ;
 
     class DateJoinVisitorGetter {
@@ -482,6 +414,7 @@ namespace dplyr{
 
     class DateJoinVisitor : public JoinVisitor, public comparisons<REALSXP>{
     public:
+        typedef NumericVector Vec ;
         typedef comparisons<REALSXP> Compare ;
         typedef boost::hash<double> hasher ;
 
@@ -520,28 +453,23 @@ namespace dplyr{
         }
 
         inline SEXP subset( const std::vector<int>& indices ) {
-            int n = indices.size() ;
-            NumericVector res = no_init(n) ;
-            for( int i=0; i<n; i++) {
-                res[i] = get(indices[i]) ;
-            }
+            NumericVector res = Subsetter<DateJoinVisitor>(*this).subset(indices) ;
             res.attr("class") = "Date" ;
             return res ;
         }
 
         inline SEXP subset( const VisitorSetIndexSet<DataFrameJoinVisitors>& set ) {
-            int n = set.size() ;
-            NumericVector res = no_init(n) ;
-            VisitorSetIndexSet<DataFrameJoinVisitors>::const_iterator it=set.begin() ;
-            for( int i=0; i<n; i++, ++it) {
-                res[i] = get(*it) ;
-            }
+            NumericVector res = Subsetter<DateJoinVisitor>(*this).subset(set) ;
             res.attr("class") = "Date" ;
             return res ;
         }
 
-        inline void print(int i){
-            Rcpp::Rcout << get(i) << std::endl ;
+        inline double get( int i) const {
+            if( i>= 0 ){
+                return left->get(i) ;
+            } else {
+                return right->get(-i-1) ;
+            }
         }
 
     private:
@@ -551,13 +479,7 @@ namespace dplyr{
 
         DateJoinVisitor( const DateJoinVisitor& ) ;
 
-        inline double get( int i) {
-            if( i>= 0 ){
-                return left->get(i) ;
-            } else {
-                return right->get(-i-1) ;
-            }
-        }
+
     } ;
 
 
