@@ -4,10 +4,11 @@ df <- expand.grid(a = 1:10, b = letters[1:10],
   KEEP.OUT.ATTRS = FALSE,
   stringsAsFactors = FALSE)
 
-srcs <- temp_srcs(c("df", "dt", "sqlite", "postgres"))
-tbls <- temp_load(srcs, df)
+tbls <- test_load(df)
 
 test_that("filter results independent of data tbl (simple)", {
+  skip_if_no_sqlite()
+
   expected <- df[df$a > 6, , drop = FALSE]
   compare_tbls(tbls[c("df","sqlite")], function(x) {
     filter_(x, ~ a > 6)
@@ -37,6 +38,11 @@ test_that("filter gives useful error message when given incorrect input", {
   expect_error( filter(tbl_df(mtcars), x ), "unknown column" )
 })
 
+test_that("filter complains in inputs are named", {
+  expect_error(filter(mtcars, x = 1), "takes unnamed arguments")
+  expect_error(filter(mtcars, x = 1 & y > 2), "takes unnamed arguments")
+})
+
 test_that("filter handles passing ...", {
   df <- data.frame( x = 1:4 )
 
@@ -165,17 +171,13 @@ test_that( "$ does not end call traversing. #502", {
 })
 
 test_that( "GroupedDataFrame checks consistency of data (#606)", {
-  df1 <- data.frame(
-   group = factor(rep(c("C", "G"), 5)),
-   value = 1:10)
-  df1 <- df1 %>% group_by(group) #df1 is now tbl
-  df2 <- data.frame(
-     group = factor(rep("G", 10)),
-     value = 11:20)
-  df3 <- rbind(df1, df2) #df2 is data.frame
-
-  expect_error( df3 %>% filter(group == "C"), "corrupt 'grouped_df', contains" )
+  df1 <- data_frame(
+   g = rep(1:2, each = 5),
+   x = 1:10
+  ) %>% group_by(g)
+  attr(df1, "group_sizes") <- c(2, 2)
 
+  expect_error(df1 %>% filter(x == 1), "corrupt 'grouped_df'" )
 })
 
 test_that( "filter uses the white list (#566)", {
@@ -188,7 +190,10 @@ X
 ")
 
   datesDF$X <- as.POSIXlt(datesDF$X)
-  expect_error(filter(datesDF, X > as.POSIXlt("2014-03-13")), "has unsupported type")
+  expect_error(
+    filter(datesDF, X > as.POSIXlt("2014-03-13")),
+    "column 'X' has unsupported class|POSIXct, not POSIXlt.*'X'"
+  )
 })
 
 test_that( "filter handles complex vectors (#436)", {
@@ -227,19 +232,10 @@ test_that("hybrid evaluation handles $ correctly (#1134)", {
   expect_equal( nrow(res), 9L )
 })
 
-# data.table --------------------------------------------------------------
-
-test_that("filter succeeds even if column called V1 (#615)", {
-  dt <- data.table::data.table(x = 1:10 ,V1 = 0)
-  out <- dt %>% group_by(V1) %>% filter(x > 5)
-
-  expect_equal(nrow(out), 5)
-})
-
 test_that("filter correctly handles empty data frames (#782)", {
   res <- data_frame() %>% filter(F)
   expect_equal( nrow(res), 0L )
-  expect_true( is.null(names(res)) )
+  expect_equal( length(names(res)), 0L )
 })
 
 test_that("filter(.,TRUE,TRUE) works (#1210)", {
@@ -290,17 +286,106 @@ test_that("grouped filter handles indices (#880)", {
   expect_equal( attr(res, "indices"), attr(res2, "indices") )
 })
 
-test_that("filter_ works (#906)", {
-  dt <- data.table::data.table(x = 1:10 ,V1 = 0)
-  out <- dt %>% filter_(~x > 5)
-  expect_equal(nrow(out), 5)
-})
-
 test_that("filter(FALSE) drops indices", {
   out <- mtcars %>%
     group_by(cyl) %>%
     filter(FALSE) %>%
     attr("indices")
   expect_equal(out, NULL)
+})
+
+test_that("filter handles S4 objects (#1366)", {
+  env <- environment()
+  Numbers <- suppressWarnings( setClass("Numbers", slots = c(foo = "numeric"), contains = "integer", where = env) )
+  on.exit(removeClass("Numbers", where = env))
+
+  df <- data.frame( x = Numbers( 1:10, foo = 10 ) )
+  res <- filter( df, x > 3 )
+  expect_true( isS4(res$x) )
+  expect_is( res$x, "Numbers")
+  expect_equal( res$x@foo, 10)
+})
+
+test_that("hybrid lag and default value for string columns work (#1403)", {
+  res <- mtcars %>%
+    mutate(xx=LETTERS[gear]) %>%
+    filter(xx==lag(xx, default='foo'))
+  xx <- LETTERS[ mtcars$gear ]
+  ok <- xx == lag( xx, default = "foo" )
+  expect_equal( xx[ok], res$xx )
+
+  res <- mtcars %>%
+    mutate(xx=LETTERS[gear]) %>%
+    filter(xx==lead(xx, default='foo'))
+  xx <- LETTERS[mtcars$gear ]
+  ok <- xx == lead( xx, default = "foo" )
+  expect_equal( xx[ok], res$xx )
+})
+
+test_that("filter recognizes global #1469", {
+    vs <- 4
+    res <- mtcars %>% filter(cyl == global(vs))
+    expect_true( all(res$cyl == 4) )
+
+    res <- mtcars %>% group_by(cyl) %>% filter(cyl == global(vs))
+    expect_true( all(res$cyl == 4) )
+})
+
+test_that("filter understands column. #1012", {
+    ir1 <- filter( iris, Sepal.Length < 5)
+    ir2 <- filter( iris, column("Sepal.Length") < 5)
+    ir3 <- filter( iris, column(paste0("Sepal.", "Length")) < 5)
+    ir4 <- filter( iris, column(~Sepal.Length) < 5 )
+    ir5 <- filter( iris, column(quote(Sepal.Length)) < 5 )
+    symb <- quote(Sepal.Length)
+    ir6 <- filter( iris, column(symb) < 5 )
+    expect_equal(ir1, ir2)
+    expect_equal(ir1, ir3)
+    expect_equal(ir1, ir4)
+    expect_equal(ir1, ir5)
+    expect_equal(ir1, ir6)
+    expect_error( filter(iris, column(~Sepal.Length + Species) < 5), "unhandled formula in column")
+    expect_error( filter(iris, column(~foo) < 5), "expands to a symbol that is not a variable from the data")
+    expect_error( filter(iris, column(letters) < 5), "column must return a single string" )
+
+    ir1 <- filter( group_by(iris, Species), Sepal.Length < 5)
+    ir2 <- filter( group_by(iris, Species), column("Sepal.Length") < 5)
+    ir3 <- filter( group_by(iris, Species), column(paste0("Sepal.", "Length")) < 5)
+    ir4 <- filter( group_by(iris, Species), column(~Sepal.Length) < 5)
+    ir5 <- filter( group_by(iris, Species), column(quote(Sepal.Length)) < 5)
+    ir6 <- filter( group_by(iris, Species), column(symb) < 5)
+    expect_equal(ir1, ir2)
+    expect_equal(ir1, ir3)
+    expect_equal(ir1, ir4)
+    expect_equal(ir1, ir5)
+    expect_equal(ir1, ir6)
+
+    expect_error( iris %>% group_by(Species) %>% filter(column(~Sepal.Length + Species) < 5 ), "unhandled formula in column")
+    expect_error( iris %>% group_by(Species) %>% filter(column(~foo) < 5), "expands to a symbol that is not a variable from the data")
+    expect_error( iris %>% group_by(Species) %>% filter(column(letters) < 5), "column must return a single string" )
 
 })
+
+
+test_that("each argument gets implicit parens", {
+  df <- data_frame(
+    v1 = c("a", "b", "a", "b"),
+    v2 = c("b", "a", "a", "b"),
+    v3 = c("a", "b", "c", "d")
+  )
+
+  tbls <- test_load(df)
+
+  one <- tbls %>% lapply(. %>% filter((v1 == "a" | v2 == "a") & v3 == "a"))
+  two <- tbls %>% lapply(. %>% filter(v1 == "a" | v2 == "a", v3 == "a"))
+
+  lapply(seq_along(one), function(i) {
+    expect_equal(collect(one[[i]]), collect(two[[i]]))
+  })
+})
+
+test_that("filter fails gracefully on raw columns (#1803)", {
+  df <- data_frame(a = 1:3, b = as.raw(1:3))
+  expect_error( filter(df, a == 1), "unsupported type" )
+  expect_error( filter(df, b == 1), "unsupported type" )
+})
