@@ -139,10 +139,22 @@ test_that("univariate left join has all columns, all rows", {
   expect_equal(j2$z.y, c(1, 2, 3, 3, NA))
 })
 
+test_that("can control suffixes with suffix argument", {
+  j1 <- inner_join(e, f, "x", suffix = c("1", "2"))
+  j2 <- left_join(e, f, "x", suffix = c("1", "2"))
+  j3 <- right_join(e, f, "x", suffix = c("1", "2"))
+  j4 <- full_join(e, f, "x", suffix = c("1", "2"))
+
+  expect_named(j1, c("x", "z1", "z2"))
+  expect_named(j2, c("x", "z1", "z2"))
+  expect_named(j3, c("x", "z1", "z2"))
+  expect_named(j4, c("x", "z1", "z2"))
+})
+
 test_that("inner_join does not segfault on NA in factors (#306)", {
   a <- data.frame(x=c("p", "q", NA), y=c(1, 2, 3), stringsAsFactors=TRUE)
   b <- data.frame(x=c("p", "q", "r"), z=c(4,5,6), stringsAsFactors=TRUE)
-  res <- inner_join(a, b, "x")
+  expect_warning(res <- inner_join(a, b, "x"), "joining factors with different levels")
   expect_equal( nrow(res), 2L )
 })
 
@@ -169,12 +181,6 @@ test_that("join handles type promotions #123", {
   res <- semi_join(df, match, c("V1", "V2"))
   expect_equal( res$V2, 3:4 )
   expect_equal( res$V3, c(103L, 109L) )
-
-  df1 <- data.frame( a = c("a", "b" ), b = 1:2, stringsAsFactors = TRUE )
-  df2 <- data.frame( a = c("a", "b" ), c = 4:5, stringsAsFactors = FALSE )
-  res <- semi_join( df1, df2, "a" )
-  res <- semi_join( df2, df1, "a" )
-
 })
 
 test_that("indices don't get mixed up when nrow(x) > nrow(y). #365",{
@@ -189,11 +195,11 @@ test_that("indices don't get mixed up when nrow(x) > nrow(y). #365",{
 test_that("join functions error on column not found #371", {
   expect_error(
     left_join(data.frame(x=1:5), data.frame(y=1:5), by="x"),
-    "cannot join on columns 'x'"
+    "column not found in lhs"
   )
   expect_error(
     left_join(data.frame(x=1:5), data.frame(y=1:5), by="y"),
-    "cannot join on columns 'y'"
+    "column not found in rhs"
   )
   expect_error(
     left_join(data.frame(x=1:5), data.frame(y=1:5)),
@@ -201,56 +207,12 @@ test_that("join functions error on column not found #371", {
   )
 })
 
-test_that("joining data tables yields a data table (#470)", {
-  a <- data.table::data.table(x = c(1, 1, 2, 3), y = 1:4)
-  b <- data.table::data.table(x = c(1, 2, 2, 4), z = 1:4)
-
-  out <- left_join(a, b, "x")
-  expect_is(out, "data.table")
-  out <- semi_join(a, b, "x")
-  expect_is(out, "data.table")
-})
-
-test_that("joining data tables does not modify them (#659)", {
-  a <- data.table::data.table(x = c(1, 1, 2, 3), y = 4:1)
-  b <- data.table::data.table(x = c(1, 2, 2, 4), z = 1:4)
-
-  for (ak in names(a)) {
-    for (bk in names(b)) {
-      label <- sprintf("keys: %s, %s", ak, bk)
-      data.table::setkeyv(a, ak)
-      data.table::setkeyv(b, bk)
-      ac <- data.table::copy(a)
-      bc <- data.table::copy(b)
-
-      out <- left_join(a, b, "x")
-      expect_equal(a, ac, label = label)
-      expect_equal(b, bc, label = label)
-      out <- semi_join(a, b, "x")
-      expect_equal(a, ac, label = label)
-      expect_equal(b, bc, label = label)
-      out <- right_join(a, b, "x")
-      expect_equal(a, ac, label = label)
-      expect_equal(b, bc, label = label)
-      out <- full_join(a, b, "x")
-      expect_equal(a, ac, label = label)
-      expect_equal(b, bc, label = label)
-      out <- inner_join(a, b, "x")
-      expect_equal(a, ac, label = label)
-      expect_equal(b, bc, label = label)
-      out <- anti_join(a, b, "x")
-      expect_equal(a, ac, label = label)
-      expect_equal(b, bc, label = label)
-    }
-  }
-})
-
 test_that("inner_join is symmetric (even when joining on character & factor)", {
   foo <- data_frame(id = factor(c("a", "b")), var1 = "foo")
   bar <- data_frame(id = c("a", "b"), var2 = "bar")
 
-  tmp1 <- inner_join(foo, bar, by="id")
-  tmp2 <- inner_join(bar, foo, by="id")
+  expect_warning(tmp1 <- inner_join(foo, bar, by="id"), "joining factor and character")
+  expect_warning(tmp2 <- inner_join(bar, foo, by="id"), "joining character vector and factor")
 
   expect_is(tmp1$id, "character")
   expect_is(tmp2$id, "character")
@@ -336,12 +298,12 @@ test_that("JoinStringFactorVisitor and JoinFactorStringVisitor handle NA #688",
   y <- data.frame(Greek = c("Alpha", "Beta", "Gamma"),
                         Letters = c("C", "B", "C"), stringsAsFactors = F)
 
-  res <- left_join(x, y, by = "Greek")
+  expect_warning(res <- left_join(x, y, by = "Greek"), "joining character vector")
   expect_true( is.na(res$Greek[3]) )
   expect_true( is.na(res$Letters[3]) )
   expect_equal( res$numbers, 1:3 )
 
-  res <- left_join(y, x, by="Greek" )
+  expect_warning(res <- left_join(y, x, by="Greek" ), "joining factor")
   expect_equal( res$Greek, y$Greek)
   expect_equal( res$Letters, y$Letters )
   expect_equal( res$numbers[1:2], 1:2 )
@@ -418,25 +380,25 @@ test_that("join can handle multiple encodings (#769)", {
   y <- data_frame(name=c("\xC9lise","Pierre","Fran\xE7ois"),attendance=c(8,10,9))
   res <- left_join(x, y, by = "name")
   expect_equal( nrow(res), 3L)
-  expect_equal( res$x, x$x)
+  expect_equal( res$name, x$name)
 
   x <- data_frame(name=factor(c("\xC9lise","Pierre","Fran\xE7ois")),score=c(5,7,6))
   y <- data_frame(name=c("\xC9lise","Pierre","Fran\xE7ois"),attendance=c(8,10,9))
   res <- suppressWarnings( left_join(x, y, by = "name") )
   expect_equal( nrow(res), 3L)
-  expect_equal( res$x, y$x)
+  expect_equal( res$name, y$name)
 
   x <- data_frame(name=c("\xC9lise","Pierre","Fran\xE7ois"),score=c(5,7,6))
   y <- data_frame(name=factor(c("\xC9lise","Pierre","Fran\xE7ois")),attendance=c(8,10,9))
   res <- suppressWarnings( left_join(x, y, by = "name") )
   expect_equal( nrow(res), 3L)
-  expect_equal( res$x, x$x)
+  expect_equal( res$name, x$name)
 
   x <- data_frame(name=factor(c("\xC9lise","Fran\xE7ois","Pierre")),score=c(5,7,6))
   y <- data_frame(name=factor(c("\xC9lise","Pierre","Fran\xE7ois")),attendance=c(8,10,9))
   res <- suppressWarnings( left_join(x, y, by = "name") )
   expect_equal( nrow(res), 3L)
-  expect_equal( res$x, x$x)
+  expect_equal( res$name, x$name)
 })
 
 test_that("join creates correctly named results (#855)", {
@@ -481,3 +443,157 @@ test_that( "ordering of strings is not confused by R's collate order (#1315)", {
   res <- a %>% inner_join(b,by=c("character"))
   expect_equal( nrow(res), 0L)
 })
+
+test_that("joins handle tzone differences (#819)", {
+  date1 <- structure(-1735660800, tzone = "America/Chicago", class = c("POSIXct", "POSIXt"))
+  date2 <- structure(-1735660800, tzone = "UTC", class = c("POSIXct", "POSIXt"))
+
+  df1 <- data.frame( date = date1 )
+  df2 <- data.frame( date = date2 )
+
+  expect_equal( attr(left_join(df1, df1)$date, "tzone"), "America/Chicago" )
+})
+
+test_that("joins matches NA in character vector (#892)", {
+  x <- data.frame(id = c(NA_character_, NA_character_),
+                   stringsAsFactors = F)
+
+  y <- expand.grid(id = c(NA_character_, NA_character_),
+                    LETTER = LETTERS[1:2],
+                    stringsAsFactors = F)
+
+  res <- left_join(x, y, by = 'id')
+  expect_true( all( is.na(res$id)) )
+  expect_equal( res$LETTER, rep(rep(c("A", "B"), each = 2), 2) )
+})
+
+test_that( "joins avoid name repetition (#1460)", {
+  d1 <- data.frame(id=1:5, foo=rnorm(5))
+  d2 <- data.frame(id=1:5, foo=rnorm(5))
+  d3 <- data.frame(id=1:5, foo=rnorm(5))
+  d <- d1 %>% left_join(d1, by="id") %>%
+    left_join(d2, by="id") %>%
+    left_join(d3, by="id")
+  expect_equal( names(d), c("id", "foo.x", "foo.y", "foo.x.x", "foo.y.y"))
+})
+
+test_that("join functions are protected against empty by (#1496)", {
+  x <- data.frame()
+  y <- data.frame(a=1)
+  expect_error( left_join(x,y, by = names(x) ), "no variable to join by" )
+  expect_error( right_join(x,y, by = names(x) ), "no variable to join by" )
+  expect_error( semi_join(x,y, by = names(x) ), "no variable to join by" )
+  expect_error( full_join(x,y, by = names(x) ), "no variable to join by" )
+  expect_error( anti_join(x,y, by = names(x) ), "no variable to join by" )
+  expect_error( inner_join(x,y, by = names(x) ), "no variable to join by" )
+})
+
+test_that("joins takes care of duplicates in by (#1192)", {
+  data2 <- data_frame(a=1:3)
+  data1 <- data_frame(a=1:3, c=3:5)
+
+  res1 <- left_join(data1, data2, by=c("a","a"))
+  res2 <- left_join(data1, data2, by=c("a" = "a"))
+  expect_equal(res1, res2)
+})
+
+# Joined columns result in correct type ----------------------------------------
+
+test_that("result of joining POSIXct is POSIXct (#1578)", {
+  data1 <- data_frame(t=seq(as.POSIXct("2015-12-01", tz="UTC"), length.out=2,
+                            by="days"), x=1:2)
+  data2 <- inner_join(data1, data1, by = "t")
+  res1 <- class(data2$t)
+  expected <- c("POSIXct", "POSIXt")
+  expect_identical(res1, expected)
+})
+
+test_that("joins allows extra attributes if they are identical (#1636)", {
+
+  tbl_left <- data_frame(
+    i = rep(c(1.0, 2.0, 3.0), each = 2),
+    x1 = letters[1:6]
+  )
+  tbl_right <- data_frame(
+    i = c(1.0, 2.0, 3.0),
+    x2 = letters[1:3]
+  )
+
+  attr(tbl_left$i, 'label') <- 'iterator'
+  attr(tbl_right$i, 'label') <- 'iterator'
+
+  res <- left_join(tbl_left, tbl_right, by = 'i')
+  expect_equal( attr(res$i, "label"), "iterator" )
+
+  attr(tbl_left$i, "foo" ) <- "bar"
+  attributes(tbl_right$i) <- NULL
+  attr(tbl_right$i, "foo" ) <- "bar"
+  attr(tbl_right$i, 'label') <- 'iterator'
+
+  res <- left_join(tbl_left, tbl_right, by = 'i')
+  expect_equal( attr(res$i, "label"), "iterator" )
+  expect_equal( attr(res$i, "foo"), "bar" )
+})
+
+test_that("joins work with factors of different levels (#1712)", {
+  d1 <- iris[, c("Species", "Sepal.Length")]
+  d2 <- iris[, c("Species", "Sepal.Width")]
+  d2$Species <- factor(as.character(d2$Species), levels=rev(levels(d1$Species)))
+  expect_warning( res1 <- left_join(d1, d2, by="Species") )
+
+  d1$Species <- as.character(d1$Species)
+  d2$Species <- as.character(d2$Species)
+  res2 <- left_join(d1, d2, by="Species")
+  expect_equal(res1, res2)
+})
+
+test_that("anti and semi joins give correct result when by variable is a factor (#1571)", {
+  big <- data.frame(letter = rep(c('a', 'b'), each = 2), number = 1:2)
+  small <- data.frame(letter = 'b')
+  expect_warning( aj_result <- anti_join(big, small, by = "letter"), NA )
+  expect_equal( aj_result$number, 1:2)
+  expect_equal( aj_result$letter, factor(c("a", "a"), levels = c("a", "b")) )
+
+  expect_warning( sj_result <- semi_join(big, small, by = "letter"), NA )
+  expect_equal( sj_result$number, 1:2)
+  expect_equal( sj_result$letter, factor(c("b", "b"), levels = c("a", "b")) )
+})
+
+test_that("inner join not crashing (#1559)", {
+  df3 <- data_frame(
+    id = c(102, 102, 102, 121),
+    name = c("qwer", "qwer", "qwer", "asdf"),
+    k = factor(c("one", "two", "total", "one"), levels = c("one", "two", "total")),
+    total = factor(c("tot", "tot", "tot", "tot"), levels = c("tot", "plan", "fact")),
+    v = c(NA_real_, NA_real_, NA_real_, NA_real_),
+    btm = c(25654.957609, 29375.7547216667, 55030.7123306667, 10469.3523273333),
+    top = c(22238.368946, 30341.516924, 52579.88587, 9541.893144)
+  )
+  df4 <- data_frame(
+    id = c(102, 102, 102, 121),
+    name = c("qwer", "qwer", "qwer", "asdf"),
+    k = factor(c("one", "two", "total", "one"), levels = c("one", "two", "total")),
+    type = factor(c("fact", "fact", "fact", "fact"), levels = c("tot", "plan", "fact")),
+    perc = c(0.15363485835208, -0.0318297270618471, 0.0466114830816894, 0.0971986553754823)
+  )
+  # all we want here is to test that this does not crash
+  expect_message( res <- replicate(100, df3 %>% inner_join(df4)) )
+  for( i in 2:100)
+    expect_equal( res[,1], res[,i] )
+})
+
+test_that( "left_join handles mix of encodings in column names (#1571)", {
+
+  df1 <- tibble::data_frame(x = 1:6, foo = 1:6)
+  names(df1)[1] <- "l\u00f8penummer"
+
+  df2 <- tibble::data_frame(x = 1:6, baz = 1:6)
+  names(df2)[1] <- iconv(  "l\u00f8penummer", from = "UTF-8", to  = "latin1" )
+
+  expect_message( res <- left_join( df1, df2 ) )
+  expect_equal( names(res), c("l\u00f8penummer", "foo", "baz") )
+  expect_equal( res$foo, 1:6)
+  expect_equal( res$baz, 1:6)
+  expect_equal( res[["l\u00f8penummer"]], 1:6)
+
+})
